
#ifndef GUARD_anyl_h
#define GUARD_anyl_h

#include "common.h"
#include "task.h"

#define ANYL_MODE_RAW 0
#define ANYL_MODE_RMSN 1
#define ANYL_MODE_PEAK 2
#define ANYL_MODE_AVG 3

#define ANYL_UNIT_HEX 0
#define ANYL_UNIT_DEC 1
#define ANYL_UNIT_VOLT 2
#define ANYL_UNIT_ENOB 3
#define ANYL_UNIT_C 4
#define ANYL_UNIT_F 5

// STOPPED -- analysis engine stopped
#define ANYL_STAT_STOPPED 0
// waiting for data from the ADC, no result
#define ANYL_STAT_NODATA 1
// collecting samples for a block, no result yet
#define ANYL_STAT_COLLECTING 2
// data is available
#define ANYL_STAT_DATA 3
// bad unit + analysis combo
#define ANYL_STAT_BADUNIT 4

// results structure as to status and logicals for status
typedef struct anyl_result_t {
	unsigned int status:3,
		newstatus:1, // includes collected field
		newresult:1,
		newdirect:1,
		floatfmt:1,
		timeout:1; // ADC timeout
	union {
		float f;
		s32 i;
	} res;
	s32 direct; // mostly for console
	u8 collected;
} anyl_result_t;

extern anyl_result_t anyl_result;

/*! Sets the display mode.  Raw=0, RMS=1, Peak=2, and Avg=3. */
void anyl_set_mode(int m);

/*! Returns the current display mode. */
int anyl_get_mode(void);

/*! Sets the number of points for average reading. */
void anyl_set_avgs(int a);

/*! Gets the current number of points used in averaging. */
u8 anyl_get_avgs(void);

/*! Sets the logic status to start taking readings and calls ads_go(). */
void anyl_go(void);

/*! Sets the logic status to stop taking readings and calls ads_stop(). */
void anyl_stop(void);

/*! Dumps old data and restarts the analysis. */
void anyl_restart(void);

void anyl_set_monitor(xTaskHandle monitor);
void anyl_set_directmonitor(xTaskHandle monitor);

/*! Initializes and sets default condition at power up.  Raw mode, averages=50 
and creates analysis task in RTOS. */
void anyl_init(int pri);
#endif
