#ifndef GUARD_dac_h
#define GUARD_dac_h

/* driver forDAC1220 on MMB3 board */

#include "common.h"
#include "task.h"
#include <signal.h>

#define BIT_16	0
#define BIT_20	1

#define MODE_NORMAL		0
#define MODE_SELFCAL	1
#define MODE_SLEEP		2

//	for DAC1220 reset pulse
#define	T1			6300				//	210:4.096MHz  6300:8MHz
#define	T2			300				//	  6:4.096MHz   300:8MHz
#define	T3			13500				//	450:4.096MHz  13500:8MHz
#define	T4			27000				//	710:4.096MHz 27000:8MHz 
#define T5			60000				//		     60000:8MHz	
#define TSC			420000				// self-cal time 500ms
								//   120000:4.096MHz 420000:8MHz

//INSR bit fields
#define DAC_WRITE	0
#define DAC_ONE_BYTE	0
#define DAC_TWO_BYTES	32
#define DAC_THREE_BYTES	64

#define DAC_DIR_REG		0
#define DAC_DIR_BYTE2	0
#define DAC_DIR_BYTE1	1
#define DAC_DIR_BYTE0	2

#define DAC_CMR_REG		4
#define DAC_CMR_BYTE1	4
#define DAC_CMR_BYTE0	5

#define DAC_OCR_REG		8
#define DAC_OCR_BYTE2	8
#define DAC_OCR_BYTE1	9
#define DAC_OCR_BYTE0	10


/* Bit bang control of DAC as other serial ports are being used. */

/*! Reset of the DAC1220. */
void dac_reset(void);

/*! Sets the resolution.  If res=0, 16-bit or res=1, 20-bit resolution. */
void dac_set_resolution(u8 res);

/*! Self calibration of the DAC. */
void dac_self_calibrate(void);

/*! Sets the operation mode of the DAC1220.  Mode=0, normal, mode=1, self calibrate and 
mode=2, sleep mode.  Resolution can also be set to 16 or 20 bit. */
void dac_set_op_mode(u8 res, u8 mode);

/*! Intializes the DAC by resetting, setting the resolution to 20 bit, self 
calibrating, then run as normal mode. */
void dac_init(void);

/*! Function writes to any register in the DAC1220. It takes a 32-bit number
as input - note that for DAC output, the number must be left-justified! The 
dac_output function handles formatting this for you.  Insr is the command byte, 
and regval the value sent. */
void dac_write(u8 insr, s32 regval);

/*! Sends a 32-bit word as output data for the DAC1220. */
void dac_output(s32 dacval);


void InitTables(int);

#endif
