
#include "flash.h"
#include "config.h"
#include "disp.h"

#define FLASH_FLAG_ADR (0x1002)
#define FLASH_CHECKSUM_ADR (0x1000)
#define FLASH_FLAG_VALUE (0xABCD)
#define FLASH_VREF (0x1004)
#define FLASH_APARMS (0x1008)
#define FLASH_SPARMS (FLASH_APARMS+(sizeof(anyl_parms_t)))

static unsigned short flash_xor_checksum(unsigned short* data, short len)
{
	unsigned short checksum=0;
	long i;
	
	for (i=0;i<len;++i) {
		checksum^=*data;
		++data;
	}

	return checksum;
}

static unsigned short flash_checksum()
{
	return ;//flash_xor_checksum((unsigned short *)(0x1002),
		//(FLASH_SPARMS+sizeof(scale_parms_t)-0x1002)/2);
}

static unsigned short flash_ram_checksum()
{
	unsigned short checksum=0;

	checksum^=flash_xor_checksum((unsigned short*)(&aparms),sizeof(anyl_parms_t)/2);	
	//checksum^=flash_xor_checksum((unsigned short*)(&sparms),sizeof(scale_parms_t)/2);	
	checksum^=flash_xor_checksum((unsigned short*)(&disp_vref),sizeof(float)/2);	
	checksum^=FLASH_FLAG_VALUE;

	return checksum;
}

void flash_read(short adr, short* data, short len)
{
	short i;
	
	for (i=0;i<len;++i) {
		*data=*((short*)(adr));
		++data;
		adr+=2;
	}
}

short flash_read_word(short adr)
{
	return *((short*)(adr));
}

int flash_load_parms()
{
	if ((flash_read_word(FLASH_FLAG_ADR)!=FLASH_FLAG_VALUE)
		|| (flash_checksum()!=flash_read_word(FLASH_CHECKSUM_ADR)))
	{
		return -1;
	}

	flash_read(FLASH_APARMS,(short*)(&aparms),sizeof(anyl_parms_t)/2);
	//flash_read(FLASH_SPARMS,(short*)(&sparms),sizeof(scale_parms_t)/2);
	flash_read(FLASH_VREF,(short*)(&disp_vref),sizeof(float)/2);

	return 0;
}

void flash_write_word(short adr, short val)
{
	short value=val;
	flash_write(adr,&value,1);
}

void flash_save_parms()
{
	flash_erase_info();

	flash_write(FLASH_APARMS,(short*)(&aparms),sizeof(anyl_parms_t)/2);
	//flash_write(FLASH_SPARMS,(short*)(&sparms),sizeof(scale_parms_t)/2);
	flash_write(FLASH_VREF,(short*)(&disp_vref),sizeof(float)/2);
	flash_write_word(FLASH_FLAG_ADR,FLASH_FLAG_VALUE);
	flash_write_word(FLASH_CHECKSUM_ADR,flash_ram_checksum());
}

void flash_init()
{
	/* FCTL2:
	FSSELx = 01 (MCLK) (MCLK must be about 7.9MHz)
	FNx = 21d (divisor = 22 -- gives about 363kHz)
	*/
	FCTL2=0xA555;
}

void flash_erase(short adr)
{
	// note: we do not use the WDT in this firmware

	FCTL3=FWKEY; // unlock
	FCTL1=FWKEY|ERASE; // erase segment
	FCTL2=0xA555;
	*((short *)(adr))=0; // dummy write
	// no need to busywait -- we are executing from flash,
	// so the cpu stops during erase
	FCTL3=FWKEY|LOCK; // lock
}

void flash_erase_info()
{
	flash_erase(0x1000); // first info segment
	flash_erase(0x1080); // second info segment
}

/*
This routine writes *words* to flash, not bytes.  (If we wrote bytes we 
could exceed tCPT.)

adr is byte address.
data is a pointer to short.
len is number of shorts (not chars)

To write e.g. structs, you must pass sizeof(struct)/2 for len.
*/
void flash_write(short adr, short* data, short len)
{
	short i;

	FCTL3=FWKEY; // unlock flash controller
	FCTL2=0xA555;
	FCTL1=FWKEY|WRT; // segment write mode

	for (i=0;i<len;++i) {
		(*((short*)(adr+(i*2))))=*data;
		// cpu stops here during write
		++data;
	}
	
	FCTL1=FWKEY; // clear write mode
	FCTL3=FWKEY|LOCK; // lock flash controller
}
