
#include "joystick.h"
#include "queue.h"

xTaskHandle js_task_handle;
#define this js_task_handle
static xTaskHandle _monitor;
static xQueueHandle _q;
static u16 _times[5]; // 1 = pressed, 0 = not pressed
static u8 _modesw; // hi bit = side, low = count
static u8 _curmodesw; // last event generated


#define JS_INTERVAL_MSEC 25.0
#define JS_HOLD_TIME_MSEC 1000

#define JS_INTERVAL ((int)(JS_INTERVAL_MSEC/(float)portTICK_RATE_MS))
#define JS_HOLD_TIME (JS_HOLD_TIME_MSEC/JS_INTERVAL_MSEC)

static void js_gen(int pressed, int jssw, int held)
{
	js_evt e;
	
	e.pressed=pressed?1:0;
	e.jssw=jssw;
	e.held=held;
	xQueueSend(_q,&e,10);
	/* doesn't do anything unless task is suspended */
	if (_monitor)
		vTaskResume(_monitor);
}

js_evt js_get(void)
{
	js_evt code={0,JS_NONE,0};

	xQueueReceive(_q,&code,0);
	return code;	
}

void js_flush(void)
{
	while (js_have())
		js_get();
}

void js_waitrelease(void)
{
	int i;
	js_evt e;

	if (!js_anypressed()) return;
	for(;;) {
		while (js_have()) {	// something in the queue
			e=js_get();
			if (e.jssw==JS_MODE_AN||e.jssw==JS_MODE_SC)
				continue;
			if (!js_anypressed()) return;
		}
		vTaskDelay(10);
	}
}

int js_anypressed(void)
{
	int i;

	for (i=1;i<=5;++i)
		if (js_pressed(i)) return 1;
	return 0;
}

js_evt js_block(void)
{
	js_evt code={0,JS_NONE,0};
	
	while (!xQueueReceive(_q,&code,1000));
	return code;
}

int js_have(void)
{
	return uxQueueMessagesWaiting(_q)!=0;
}

static void js_update(int jssw)
{
	int pressed=(~P1IN>>3)&(1<<(jssw-1));
	u16 t=_times[jssw-1];

	if (pressed) {
		if (!(t&0x8000)) {
			t=0x8000;
		} else {
			++t;
			if (t==0xFFFF) --t;
			if (t==0x8001) {
				js_gen(1,jssw,0);
			} else if (t==0x8001+JS_HOLD_TIME) {
				js_gen(1,jssw,1);
			}
		}
	} else {
		if (t&0x8000) {
			t=0;
		} else {
			++t;
			if (t==0x7fff) --t;
			if (t==1)
				js_gen(0,jssw,0);
		}
	}
	_times[jssw-1]=t;
}

int js_modesw(void)
{
	return JS_MODE_AN;
}



int js_pressed(int jssw)
{
	if (!jssw) return 0;
	return _times[jssw-1]>=0x8001;
}

static void js_task(void *p)
{
	int i;
	portTickType lastwake;

	lastwake=xTaskGetTickCount();
	for(;;) {
		for (i=1;i<=5;++i)
			js_update(i);
		vTaskDelayUntil(&lastwake,JS_INTERVAL);
	}
}

void js_set_monitor(xTaskHandle monitor)
{
	_monitor=monitor;
}

void js_init(u8 pri)
{
	int i;

	_monitor=0;
	_q=xQueueCreate(5,sizeof(js_evt));
	for(i=0;i<5;++i) _times[i]=0;
	xTaskCreate(js_task,"js",40,0,pri,&this);
}
