
#include "lcd.h"

/* Starburst patterns originally based on a demo from Rowley & Assoc. */

// Starburst segment definitions.
#define _D 0x0001
#define _C 0x0002
#define _B 0x0004
#define _A 0x0008
#define _E 0x0020
#define _F 0x0040
#define _P 0x0100
#define _Q 0x0200
#define _G 0x0400
#define _H 0x0800
#define _N 0x1000
#define _M 0x2000
#define _K 0x4000
#define _J 0x8000


static const u16 starburst_segments[] =
{
  /* */   0,
  /*!*/   _F | _E,    // Lame
  /*"*/   _H | _K,
  /*#*/   _J | _B | _P | _C | _G | _M,
  /*$*/   _A | _F | _G | _M | _C | _D | _J | _P,
  /*%*/   _K|_Q|_C|_F,
  /*&*/   _A|_F|_G|_M|_E|_P|_D,
  /*'*/   _K,
  /*(*/   _K|_N,
  /*)*/   _H|_Q,
  /***/   _H | _J | _K | _G | _M | _Q | _P | _N,
  /*+*/   _J | _G | _M | _P,
  /*,*/   _Q,
  /*-*/   _G | _M,
  /*.*/   0,          // uses decimal point
  /*/*/   _K | _Q,
  /*0*/   _A | _B | _C | _D | _E | _F | _Q | _K,
  /*1*/   _C | _B | _K,
  /*2*/   _A | _B | _M | _G | _E | _D,
  /*3*/   _A | _B | _M | _G | _C | _D,
  /*4*/   _F | _G | _M | _B | _C,
  /*5*/   _A | _F | _G | _M | _C | _D,
  /*6*/   _A | _F | _E | _D | _C | _M | _G,
  /*7*/   _A | _B | _C | _M,
  /*8*/   _A | _B | _C | _D | _E | _F | _G | _M,
  /*9*/   _A | _F | _G | _M | _B | _C | _D,
  /*:*/   0,          // No good idea
  /*;*/   0,          // No good idea
  /*<*/   _K | _N,
  /*=*/   _G | _M | _D,
  /*>*/   _H | _Q,
  /*?*/   _A | _B | _M | _G | _E,
  /*@*/   _A | _B | _C | _D | _E | _F |_M |_P,
  /*A*/   _E | _F | _A | _B | _C | _G | _M,
  /*B*/   _A | _B | _C | _D | _J | _P | _M,
  /*C*/   _A | _F | _E | _D,
  /*D*/   _A | _B | _C | _D | _J | _P,
  /*E*/   _A | _F | _E | _D | _G | _M,
  /*F*/   _A | _F | _E | _G | _M,
  /*G*/   _A | _F | _E | _D | _C | _M,
  /*H*/   _F | _E | _G | _M | _B | _C,
  /*I*/   _A | _J | _P | _D,
  /*J*/   _B | _C | _D | _E,
  /*K*/   _F | _E | _G | _K | _N,
  /*L*/   _F | _E | _D,
  /*M*/   _E | _F | _H | _K | _B | _C,
  /*N*/   _E | _F | _H | _N | _C | _B,
  /*O*/   _A | _B | _C | _D | _E | _F,
  /*P*/   _E | _F | _A | _B | _G | _M,
  /*Q*/   _A | _B | _C | _D | _E | _F | _N,
  /*R*/   _E | _F | _A | _B | _G | _M | _N,
  /*S*/   _A | _F | _G | _M | _C | _D,
  /*T*/   _A | _J | _P,
  /*U*/   _F | _E | _D | _C | _B,
  /*V*/   _F | _E | _Q | _K,
  /*W*/   _F | _E | _Q | _N | _C | _B,
  /*X*/   _H | _K | _Q | _N,
  /*Y*/   _H | _K | _P,
  /*Z*/   _A | _K | _Q | _D,
  /*[*/   _A | _F | _E | _D,
  /*\*/   _K | _Q,
  /*]*/   _A | _B | _C | _D,
  /*^*/   _H | _B,
  /*_*/   _D,
  /*`*/   _F,
  /*a*/   _A | _B | _C | _D | _E | _G | _M,
  /*b*/   _F | _E | _D | _C | _G | _M,
  /*c*/   _G | _M | _E | _D,
  /*d*/   _G | _M | _E | _D | _C | _B,
  /*e*/   _A|_B|_F|_E|_D|_G|_M,
  /*f*/   _A|_F|_E|_G|_M,
  /*g*/   _A | _F | _E | _D | _C | _M,
  /*h*/   _F|_E|_C|_G|_M,
  /*i*/   _J|_P,
  /*j*/   _B | _C | _D | _E,
  /*k*/   _F | _E | _G | _K | _N,
  /*l*/   _J|_P,
  /*m*/   _E|_G|_M|_P|_C,
  /*n*/   _E|_G|_M|_C,
  /*o*/   _E|_G|_M|_D|_C,
  /*p*/   _E | _F | _A | _B | _G | _M,
  /*q*/   _A|_F|_B|_G|_M|_C,
  /*r*/   _E|_G|_M,
  /*s*/   _A | _F | _G | _M | _C | _D,
  /*t*/   _F|_E|_D|_G|_M,
  /*u*/   _E|_D|_C,
  /*v*/   _F | _E | _Q | _K,
  /*w*/   _E|_D|_P|_C,
  /*x*/   _H | _K | _Q | _N,
  /*y*/   _F | _G | _M | _B | _C|_D,
  /*z*/   _A | _K | _Q | _D,
  /*{*/   _A|_D|_H|_J|_G|_Q|_P,
  /*|*/   _J|_P,
  /*}*/   _A|_D|_J|_K|_P|_N|_M,
  /*~*/   _H|_K,
  /*DEL*/ _A|_F|_B|_G|_M // degree symbol
};

static const u16 bgtbl[]={
	0x0801,0x0C01,0x0E01,0x0F01,0x0F11,0x0F31,0x0F71,0x0FF1,0x0FF9,0x0FFD,0x0FFF
};

char dispbuf[10]; // global, for convenience

void lcd_arrowset(u8 arrow, u8 show)
{
	volatile u8 *adr;
	
	adr = &LCDM15;
	
	if (arrow>3) return;
	// if show = 1 leave current settings, if show = 0 clear all arrows
	if (!show){
	*adr &= 0xF0;	//clears all arrows
	}
	// arrow definition: Left = 0, Up = 1, Right = 2, Down = 3
	*adr |= 1<<arrow;
}

void lcd_antenna(u8 show)
{
	volatile u8 *adr;
	
	adr = &LCDM17;
	
	// show = 1, antenna is on; show = 0 antenna is off
	if (show)
		*adr |= 0x20;
	else
		*adr &= 0xDF;
	
}

void lcd_ss(u8 strength)
{
	volatile u8 *adr;
	
	adr = &LCDM17;
	
	//strength goes from 0 to 3; 0 turns indicator off, 1-3 light up segments
	if(strength>3) return;
	
	*adr &= 0x2F;
	switch (strength)
	{
		case 0:
			break;
		case 1:
			*adr |= 0x10;
			break;
		case 2:
			*adr |= 0x90;
			break;
		case 3:
			*adr |= 0xD0;
			break;
	}
}

void lcd_battery_show(u8 show)
{
	volatile u8 *adr;
	
	adr = &LCDM15;
	
	// show = 1, battery indicator is on; show = 0 battery indicator is off
	if (show)
		*adr |= 0x10;
	else
		*adr &= 0x0F;
}

void lcd_battery_set(u8 value)
{
	volatile u8 *adr;
	
	adr = &LCDM15;
	
	// value goes from 0 to 3; 0 turns bars off, 1-3 light up segments
	if(value>3) return;
	
	*adr &= 0x1F;
	switch (value)
	{
		case 0:
			break;
		case 1:
			*adr |= 0x90;
			break;
		case 2:
			*adr |= 0xD0;
			break;
		case 3:
			*adr |= 0xF0;
			break;
	}

}

void lcd_bargraph_show(u8 show)
{
	volatile u8 *adr;
	
	adr = (u8 *)&LCDM17;
	
	// show = 1, bargraph is on; show = 0 bargraph is off
	if (show)
	{
		*adr |= 0x08;
		--adr;
		*adr |= 0x01;
	}
	else
	{
		*adr &= 0xF0;
		--adr;
		*adr = 0;
	}

}

void lcd_bargraph_set(u8 value)
{
	volatile char *base;
	
	base = &LCDM16;
	
	//clear bargraph
	base[0] &= 0x01;
	base[1] &= 0xF8;
	if (value<11){
	// value goes from 0 to 10; 0 turns bars off, 1-10 light up segments
	base[0] = bgtbl[value];
	base[1] = bgtbl[value]>>8 | base[1];
	}
}

void lcd_showc(char c, int position)
{
	volatile char *adr;
	u16 segs;
	
	if (position<1) return;
	if (position>7) return;
	adr=&LCDM1+((position-1)*2);
	if ((c&0x7f)<32) segs=0;
	else segs=starburst_segments[(c&0x7f)-32];
	
	adr[0] = segs|(adr[0]&0x90);
	adr[1] = (segs>>8); 
}

void lcd_allclear(void)
{
	volatile u8 *adr;
	
	for (adr=(u8 *)&LCDM1;adr<(u8 *)&LCDM18;++adr)
		*adr=0;
}

void lcd_clear(void)
{
	volatile int count;
	count=1;
	while(count<7)
	{
		lcd_dp_set(count,0);
		++count;
	}
	lcd_showstr_pos("       ", 1);
	
}

void lcd_allon(void)
{
	volatile u8 *adr;
	
	for (adr=(u8 *)&LCDM1;adr<(u8 *)&LCDM18;++adr)
		*adr=0xff;
}

void lcd_dp_set(int ind, int on)
{
	volatile u8 *adr;

	if (ind<1||ind>7) return;
	adr=(u8 *)(&LCDM3+((ind-1)*2));
	if (on)
		*adr|=0x10;
	else
		*adr&=~0x10;
}


static void delay(unsigned long i)
{
	volatile unsigned long i2;
	
	for (i2=0;i2<i;++i2);
}

void lcd_showstr_pos(char *c, int pos)
{
	volatile char *adr;
	int check=1;
	adr=&LCDM1;
	
	if (!c) return;
	while (*c) {
		if (pos>7) return;
		switch(*c)
		{
			case '+':
				adr[0]&=~0x10;
				adr[0]|=0x80;
				check=0;
				break;
			case '-':
				adr[0]&=~0x80;
				adr[0]|=0x10;
				check=0;
				break;
			case '.':
				lcd_dp_set(pos-1,1);
				break;
			default:
				if (check) adr[0]&=~0x90;
				lcd_showc(*c,pos++);
				
		}
		++c;
		
	}
}


void lcd_test(void)
{
	u8 segs;

	segs=0x10;
	for(;;) {
		*(volatile u8 *)(0x95)=segs&15;
		if (segs==0x10)
			segs=1;
		else
			segs=segs<<1;
		delay(250000);
	}
}

void lcd_init(void)
{
	// Basic Timer from ACLK
	BTCTL = 0x2C;					// LCD = 512Hz,	Timer = 4Hz
	BTCNT1 = 0;						// counter 1 clear
	BTCNT2 = 0;						// counter 2 clear
	P5SEL = 0xFF;					// P5 for LCD
	
	//lcdctl=111 11 101
	//LCDCTL = LCDON + LCD4MUX + LCDSG0_4;		// S0 - S27, 4MUX
	
	lcd_allclear();
	
	//lcdctl=1101 1101  segemnts S0-S35
	//lcdctl=1011 1101 segments S0-S31
	LCDCTL=0xbd;
	P5SEL|=0x4;
	P5SEL|=0x8;
	P5SEL|=0x10;
	P5SEL|=0x2;
	P5SEL|=0x1;
	//P4SEL|=0x80;
	//P4SEL|=0x40;
	P5SEL|=0x80;
	P5SEL|=0x40;
	P5SEL|=0x20;

}
