
#include "common.h"
#include "task.h"
#include "lcd.h"
#include "dac.h"
#include "uart.h"
#include "ads.h"
#include "i2c.h"
#include "flash.h"
#include <signal.h>


// this is in core.c, which has no header ...
void core_init(void);
// ... and this is in test.c, which also has no header
void test_init(void);
// ... in console.c, yet another with no header
void console_init(int pri);

static void delay(unsigned long i)
{
	volatile unsigned long i2;
	
	for (i2=0;i2<i;++i2);
}


int main(void)
{
	u32 i,j;
	/* May need to change MSP430 settings for oscillator.  MSP430F449 starts @1MHz and
	operates up to 8MHz. The settings are:
	The multiplier (N+1) and D set the frequency of DCOCLK.
	DCOPLUS = 0: fDCOCLK = (N + 1) x fACLK
	DCOPLUS = 1: fDCOCLK = D x (N + 1) x fACLK
	fACLK is the watch crystal frequency 32.768KHz 
	If FN_2 is 0 then SCFQCTL+1 times fACLK is MCLK.
	If FN_2 is 0x04, then same as above times 2.*/
	
	WDTCTL = WDTPW|WDTHOLD;			// Stop watchdog timer
	FLL_CTL0 = XCAP18PF;			// Configure load caps
	/* For 4MHz */
	
	//SCFI0 |= FN_2;					// for 4MHz
	//SCFQCTL = 124;					// 32768 x (124+1) = 4.096000MHz
	
	
	
	/* For 8MHz */
		/* D=2, N=121, no mod */
	SCFQCTL=121;					// 32768 x (121+1) = 7.995392 MHz
	SCFI0=0x44;
	
	FLL_CTL0=0x90;
	FLL_CTL1=0x20;
	do {
		IFG1&=~OFIFG;
		FLL_CTL1=0x20;
	} while (OFIFG&IFG1);

	
	
/*****************************************************************************
*  No further changes are required in the rest of the function               *
*								             *
*****************************************************************************/
		
	// I/O port (NC pin as OUT : Recommended to reduce supply current)
	P1DIR = 0x07;	// P1
	P2DIR = 0xDB;	// P2
	P3DIR = 0xFF;	// P3
	P4DIR = 0xF7;	// P4
	P5DIR = 0x9F;	// P5
	P6DIR = 0xFF;	// P6

	P2IES = 0x04;	// P2.2 Falling edge
	
	

	// init peripherals, and start with the LCD
	lcd_init();
	
	// show start screen, in case tasks don't work
	lcd_showstr("START");
	//lcd_allclear();
	// initialize the I2C interface
	i2cInit();
	lcd_showstr("START1");
	// initialize and calibrate the DAC to 0V output
	dac_init();
	
	DACOUTL();
	dac_output(524287);

	// adc stopped initially
	ads_init();
	
	// flash memory
	flash_init();
#ifndef ETEST
	/* check for joystick press on reset to determine if test mode is selected by stripping top for bits of
	of port 1 and looking for P1.4 to go low...the keypress is joystick down on reset */
	//if ((P1IN>>4)==14)
		//test_init();	// task is set to idle prioity
	//else
		core_init();	// task is set to idle priority
	// initiaize the joystick tasks 
	js_init(0);
	// initialize the analysis mode
	anyl_init(1);
	// intialize the serial output for communication with highest priority
	console_init(2);
#else
	anyl_init(1);
	console_init(2);

#endif

	// turn on the interrupts
	eint();

	// let's go!....by starting the RTOS task scheduler
lcd_showstr("START2");	
	vTaskStartScheduler();

	for(;;);
	return 0;
}
