
	;; misc fast BCD math routines
	;; mostly C callable
	;; see bcd.h

	;; M. P. Ashton
	;; (c) 2003 Texas Instruments Inc.

	.section .text

	.global bcd64add
	.global bcd64div2

	;; short bcd64add(void *a, void *b, short c)
	;; executes *b+=*a+c in 64 bit packed BCD
	;; returns carry (0 or 1)
	;; uses sbcd16addc
bcd64add:
	push	r11
	call	#sbcd16addc
	incd	r14
	incd	r15
	call	#sbcd16addc
	incd	r14
	incd	r15
	call	#sbcd16addc
	incd	r14
	incd	r15
	call	#sbcd16addc
	mov	r13,r15
	pop	r11
	ret

	;; void bcd64div2(void *a)
	;; divides 64-bit BCD word at a by 2
	;; division is truncating
bcd64div2:
	push	r11
	mov	r15,r14
	clrc
	rrc	6(r14)		; >>=1
	rrc	4(r14)
	rrc	2(r14)
	rrc	0(r14)
	;; do adjust & add for all words
	clr	r13		; clear carry for add routine
	call	#1f
	incd	r14
	call	#1f
	incd	r14
	call	#1f
	incd	r14
	call	#1f
	pop	r11
	ret

	;; adjust 1 16-bit word (4 digits)
	;; word address in r14
	;; clobbers r11-r13 and r15, and calls sbcd16addc
	;; uses one word on stack
	;; r14 unmodified
1:	mov	@r14,r12
	and	#0x8888,r12
	and	#0x7777,@r14
	clrc
	rrc	r12
	mov	r12,r11
	rra	r11
	rra	r11
	bis	r11,r12
	;; r12 now contains adjuster digits
	;; put r12 on stack & do add
	push	r12
	mov	r1,r15
	call	#sbcd16addc
	pop	r12
	ret

	;; sbcd16addc - executes *b+=*a+c in bcd
	;; where r15 = *a
	;; r14 = *b
	;; r13 = c (carry)
	;; carry returned in r13
	;; clobbers r11, r12
	;; r14, r15 not modified
sbcd16addc:
	;; r12=*r14+0x6666
	mov	#0x6666,r12
	add	@r14,r12
	;; r11=r12+*r15
	mov	@r15,r11
	rrc	r13
	addc	r12,r11
	;; save carry in r13
	rlc	r13
	;; r12=~(r11^(r12^*r15))&0x1111
	xor	@r15,r12
	xor	r11,r12
	inv	r12
	and	#0x1110,r12
	;; *r14=r11-((r12>>2)|(r12>>3))
	mov	r11,0(r14)
	setc			; copy carry bit
	bic	r13,r2		; from r13 bit 0, and invert
	rrc	r12
	clrc
	rrc	r12
	mov	r12,r11
	rrc	r12
	bis	r11,r12
	sub	r12,0(r14)
	ret

