
#ifndef GUARD_uart_h
#define GUARD_uart_h
/*! Creates a 8 character queue with RTOS and assigns to queue handle iq 
which is static within uart.c. 8 characters is the maximum queue length.  Characters
are placed in the queue from the UART ISR. */
void uart_init(void);

/*! Reads a line of characters and ends with a CRLF. Line of characters
are saved to string array pointed to by *s with a maximum length of len. 
Characters received are echoed back to the terminal*/
void uart_readln(char *s, int len);

/*! Calls for the next character from the queue. */
char uart_get(void);

/*! Pulls a character from the queue and places in the char variable
pointed by *c. The call returns if there isn't a character within the time specified by
timeout. */
int uart_get_timeout(char *c, int timeout);

/*! Checks to see if there is anything to get from the queue. */
int uart_have(void);

/*! Sends a string of characters out the UART pointed by *c.*/
void uart_txstr(char *c);

/*! Sends a single character c out the UART. */
void uart_txc(char c);



#endif
